package com.autentia.tutoriales.spring.daojdbc;

import org.junit.Assert;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.autentia.tutoriales.spring.daojdbc.entity.Product;
import com.autentia.tutoriales.spring.daojdbc.managers.HibernateProductoDao;
import com.autentia.tutoriales.spring.daojdbc.managers.JdbcProductoDao;

/**
 * Maven 2: Tests funcionales a realizar
 * Es sólo a nivel didactico, no se puede considerar un tests reales
 */
public class ProductosTest  {

	private ApplicationContext  factory;  

	/** 
	 * Inicializamos el contexto de Spring 
	 */  
	@org.junit.Before  
	public void initTests(){  
		this.factory  = new ClassPathXmlApplicationContext("applicationContext.xml");
	}  

	/**
	 * El siguiente test verifica la integración JDBC y Spring 
	 */
	@org.junit.Test
	public void testProductJDBC() {
		try {
			JdbcProductoDao productoDao	= (JdbcProductoDao) factory.getBean("productoDao");
			
			Product p = getRandomProduct("jdbc");
			
			// Insertamos un nuevo producto
			productoDao.insert(p);
			
			// Recuperamos el producto añadido
			p = productoDao.get(p.getCodigo());
			
			Assert.assertNotNull(p);
		} catch (org.springframework.dao.DataAccessException ex){
			Assert.fail();
		}
	}
	
	/**
	 * El siguiente test verifica la integración Hibernate 3 SIN Anotaciones y Spring 
	 */
	@org.junit.Test
	public void testProductHibernate3() {
		try {
			HibernateProductoDao productoDao = (HibernateProductoDao) factory.getBean("productoDaoNoAnotado");
			
			Product p = getRandomProduct("hib");
			
			// Insertamos un nuevo producto
			productoDao.insert(p);

			// Recuperamos el producto añadido
			p = productoDao.get(p.getCodigo());
			
			Assert.assertNotNull(p);
		} catch (org.springframework.dao.DataAccessException ex){
			Assert.fail();
		}
	}
	
	/**
	 * El siguiente test verifica la integración Hibernate 3 CON Anotaciones y Spring 
	 */
	@org.junit.Test
	public void testProductHibernate3Annotated() {
		try {
			HibernateProductoDao productoDao = (HibernateProductoDao) factory.getBean("productoDaoAnotado");
			
			Product p = getRandomProduct("hibann");
			
			
			// Insertamos un nuevo producto
			productoDao.insert(p);

			// Recuperamos el producto añadido
			p = productoDao.get(p.getCodigo());
			
			Assert.assertNotNull(p);
		} catch (org.springframework.dao.DataAccessException ex){
			Assert.fail();
		}
	}
	
	/**
	 * El siguiente test será correcto si se lanza la excepción: org.springframework.dao.DataIntegrityViolationException
	 * 
	 * OJO, Observe que en este tutorial no se trata la transaccionalidad, por lo que se añadiria sólo el primer producto.
	 * 
	 * No se pueden tener dos productos con el mismo código
	 */
	@org.junit.Test(expected=org.springframework.dao.DataIntegrityViolationException.class)
	public void testStringDataException() {
		JdbcProductoDao productoDao	= (JdbcProductoDao) factory.getBean("productoDao");
		
		Product p = getRandomProduct("jdbc");
		
		// Insertamos un nuevo producto
		productoDao.insert(p);
		productoDao.insert(p);
	}
	
	
	/**
	 * @return Devuelve un producto de ejemplo (dummy) con el que trabajar
	 */
	private Product getRandomProduct(String pre){
		String		codigo		= pre + String.valueOf(System.currentTimeMillis());
		String		descripcion = "Descripcion " + codigo;
		
		Product p = new Product();
		p.setCodigo(codigo);
		p.setDescripcion(descripcion);
		
		return p;
	}
}
